Package: sdcMicro
Type: Package
Title: Statistical Disclosure Control Methods for Anonymization of Data
        and Risk Estimation
Version: 5.8.2
Date: 2026-06-05
Authors@R: c(
  person("Matthias", "Templ", email="matthias.templ@gmail.com", role = c("aut", "cre"), comment=c(ORCID="0000-0002-8638-5276")),
  person("Bernhard", "Meindl", email = "Bernhard.Meindl@statistik.gv.at", role = c("aut")),
  person("Alexander", "Kowarik", email="Alexander.Kowarik@statistik.gv.at", role = "aut", comment=c(ORCID="0000-0001-8598-4130")),  
  person("Johannes", "Gussenbauer", email = "johannes.gussenbauer@statistik.gv.at", role = c("aut")),
  person("Organisation For Economic Co-Operation And Development", role=c("cph"), comment="Initial published c(++) code (under LGPL) code for rank swapping, mdav-microaggregation, suda2 and other (hierarchical) risk measures"),
  person("Statistics Netherlands", role=c("cph"), comment="microAggregation cpp code (under EUPL v1.1)"),
  person("Pascal Heus", role=c("cph"), comment="original measure threshold cpp code (under LGPL)"))
Description: Data from statistical agencies and other institutions are mostly
    confidential. This package, introduced in Templ, Kowarik and Meindl (2017) <doi:10.18637/jss.v067.i04>, can be used for the generation of anonymized
    (micro)data, i.e. for the creation of public- and scientific-use files.
    The theoretical basis for the methods implemented can be found in Templ (2017) <doi:10.1007/978-3-319-50272-4>.
    Various risk estimation and anonymization methods are included. Note that the package
    includes a graphical user interface published in Meindl and Templ (2019) <doi:10.3390/a12090191> that allows to use various methods of this
    package.
LazyData: TRUE
ByteCompile: TRUE
LinkingTo: Rcpp
Depends: R (>= 2.10)
Suggests: clue, laeken, parallel, testthat, pdftools, yaml
Imports: utils, stats, graphics, car, carData, rmarkdown, knitr,
        data.table, xtable, robustbase, cluster, MASS, e1071, tools,
        Rcpp, methods, ggplot2, shiny (>= 1.4.0), haven, rhandsontable,
        DT, prettydoc, VIM (>= 4.7.0), httr, jsonlite
License: GPL-2
URL: https://github.com/sdcTools/sdcMicro
Collate: '0classes.r' 'addGhostVars.R' 'addNoise.r' 'aux_functions.r'
        'createDat.R' 'createNewID.R' 'dataGen.r' 'dataSets.R'
        'dRisk.R' 'dRiskRMD.R' 'dUtility.R' 'freqCalc.r'
        'globalRecode.R' 'groupAndRename.R' 'GUIfunctions.R'
        'indivRisk.R' 'infoLoss.R' 'AI_createSdcObj.R'
        'ai_access_utilities.R' 'AI_applyAnonymization.R'
        'ai_anonymization_utilities.R' 'LocalRecProg.R' 'localSupp.R'
        'localSuppression.R' 'mdav.R' 'measure_risk.R' 'methods.r'
        'microaggregation.R' 'modRisk.R'
        'muargus_compatibility_functions.R' 'mvTopCoding.R'
        'plotFunctions.R' 'plotMicro.R' 'pram.R' 'rankSwap.R'
        'RcppExports.R' 'recordLinkage.R' 'recordSwap.R' 'report.R'
        'riskyCells.R' 'sdcMicro-package.R' 'shuffle.R' 'suda2.R'
        'timeEstimation.R' 'topBotCoding.R' 'valTable.R' 'zzz.R'
        'printFunctions.R' 'mafast.R' 'maG.R' 'sdcApp.R'
        'show_sdcMicroObj.R'
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-06-12 16:25:52 UTC; matthias
Author: Matthias Templ [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8638-5276>),
  Bernhard Meindl [aut],
  Alexander Kowarik [aut] (ORCID:
    <https://orcid.org/0000-0001-8598-4130>),
  Johannes Gussenbauer [aut],
  Organisation For Economic Co-Operation And Development [cph] (Initial
    published c(++) code (under LGPL) code for rank swapping,
    mdav-microaggregation, suda2 and other (hierarchical) risk
    measures),
  Statistics Netherlands [cph] (microAggregation cpp code (under EUPL
    v1.1)),
  Pascal Heus [cph] (original measure threshold cpp code (under LGPL))
Maintainer: Matthias Templ <matthias.templ@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-12 17:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-06-12 20:31:58 UTC; unix
Archs: sdcMicro.so.dSYM
