# Generated by roxygen2: do not edit by hand

S3method(plot,localSuppression)
S3method(plot,sdcMicroObj)
S3method(print,freqCalc)
S3method(print,il_correl)
S3method(print,il_variables)
S3method(print,indivRisk)
S3method(print,ldiversity)
S3method(print,localSuppression)
S3method(print,measure_risk)
S3method(print,micro)
S3method(print,modrisk)
S3method(print,pram)
S3method(print,recordLinkage)
S3method(print,suda2)
S3method(recordSwap,default)
S3method(recordSwap,sdcMicroObj)
S3method(summary,freqCalc)
S3method(summary,micro)
S3method(summary,pram)
export("strataVar<-")
export(AI_applyAnonymization)
export(AI_createSdcObj)
export(IL_correl)
export(IL_variables)
export(LocalRecProg)
export(addGhostVars)
export(addNoise)
export(argus_microaggregation)
export(argus_rankswap)
export(calcRisks)
export(createDat)
export(createNewID)
export(createSdcObj)
export(dRisk)
export(dRiskRMD)
export(dUtility)
export(dataGen)
export(extractManipData)
export(freq)
export(freqCalc)
export(generateStrata)
export(get.sdcMicroObj)
export(globalRecode)
export(groupAndRename)
export(importProblem)
export(indivRisk)
export(infoLoss)
export(kAnon)
export(kAnon_violations)
export(ldiversity)
export(localSupp)
export(localSuppression)
export(mafast)
export(measure_risk)
export(mergeHouseholdData)
export(microaggrGower)
export(microaggregation)
export(modRisk)
export(mvTopCoding)
export(nextSdcObj)
export(plot.localSuppression)
export(plotMicro)
export(pram)
export(rankSwap)
export(readMicrodata)
export(recordLinkage)
export(recordSwap)
export(removeDirectID)
export(report)
export(riskyCells)
export(sdcApp)
export(selectHouseholdData)
export(set.sdcMicroObj)
export(shuffle)
export(suda2)
export(topBotCoding)
export(undolast)
export(valTable)
export(varToFactor)
export(varToNumeric)
export(writeSafeFile)
exportClasses(sdcMicroObj)
exportMethods("strataVar<-")
exportMethods(kAnon_violations)
exportMethods(print)
exportMethods(show)
import(MASS)
import(Rcpp)
import(carData)
import(cluster)
import(data.table)
import(ggplot2)
import(knitr)
import(methods)
import(rhandsontable)
import(robustbase)
import(shiny)
import(tools)
import(xtable)
importFrom(DT,datatable)
importFrom(MASS,loglm)
importFrom(MASS,mvrnorm)
importFrom(data.table,data.table)
importFrom(data.table,is.data.table)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,boxplot)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(prettydoc,html_pretty)
importFrom(rmarkdown,pandoc_available)
importFrom(rmarkdown,render)
importFrom(robustbase,covMcd)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,str)
useDynLib(sdcMicro, .registration=TRUE)
